diff --git a/contrib/pg_upgrade/check.c b/contrib/pg_upgrade/check.c
new file mode 100644
index bed10f8..beb177d
*** a/contrib/pg_upgrade/check.c
--- b/contrib/pg_upgrade/check.c
*************** static void check_for_prepared_transacti
*** 21,26 ****
--- 21,27 ----
  static void check_for_isn_and_int8_passing_mismatch(ClusterInfo *cluster);
  static void check_for_reg_data_type_usage(ClusterInfo *cluster);
  static void get_bin_version(ClusterInfo *cluster);
+ static char *get_canonical_locale_name(int category, const char *locale);
  
  
  /*
*************** set_locale_and_encoding(ClusterInfo *clu
*** 359,366 ****
  		i_datcollate = PQfnumber(res, "datcollate");
  		i_datctype = PQfnumber(res, "datctype");
  
! 		ctrl->lc_collate = pg_strdup(PQgetvalue(res, 0, i_datcollate));
! 		ctrl->lc_ctype = pg_strdup(PQgetvalue(res, 0, i_datctype));
  
  		PQclear(res);
  	}
--- 360,382 ----
  		i_datcollate = PQfnumber(res, "datcollate");
  		i_datctype = PQfnumber(res, "datctype");
  
! 		if (GET_MAJOR_VERSION(cluster->major_version) < 902)
! 		{
! 			/*
! 			 *	Pre-9.2 did not canonicalize the supplied locale names
! 			 *	to match what the system returns, while 9.2+ does, so
! 			 *	convert pre-9.2 to match.
! 			 */
! 			ctrl->lc_collate = get_canonical_locale_name(LC_COLLATE,
! 							   pg_strdup(PQgetvalue(res, 0, i_datcollate)));
! 			ctrl->lc_ctype = get_canonical_locale_name(LC_CTYPE,
! 							   pg_strdup(PQgetvalue(res, 0, i_datctype)));
!  		}
! 		else
! 		{
! 	 		ctrl->lc_collate = pg_strdup(PQgetvalue(res, 0, i_datcollate));
! 			ctrl->lc_ctype = pg_strdup(PQgetvalue(res, 0, i_datctype));
! 		}
  
  		PQclear(res);
  	}
*************** get_bin_version(ClusterInfo *cluster)
*** 931,933 ****
--- 947,986 ----
  
  	cluster->bin_version = (pre_dot * 100 + post_dot) * 100;
  }
+ 
+ 
+ /*
+  * get_canonical_locale_name
+  *
+  * Send the locale name to the system, and hope we get back a canonical
+  * version.  This should match the backend's check_locale() function.
+  */
+ static char *
+ get_canonical_locale_name(int category, const char *locale)
+ {
+ 	char	   *save;
+ 	char	   *res;
+ 
+ 	save = setlocale(category, NULL);
+ 	if (!save)
+         pg_log(PG_FATAL, "failed to get the current locale\n");
+ 
+ 	/* 'save' may be pointing at a modifiable scratch variable, so copy it. */
+ 	save = pg_strdup(save);
+ 
+ 	/* set the locale with setlocale, to see if it accepts it. */
+ 	res = setlocale(category, locale);
+ 
+ 	if (!res)
+         pg_log(PG_FATAL, "failed to get system local name for \"%s\"\n", res);
+ 
+ 	res = pg_strdup(res);
+ 
+ 	/* restore old value. */
+ 	if (!setlocale(category, save))
+         pg_log(PG_FATAL, "failed to restore old locale \"%s\"\n", save);
+ 
+ 	free(save);
+ 
+ 	return res;
+ }
